@@ -2,7 +2,7 @@
 #
 # The internetarchive module is a Python/CLI interface to Archive.org.
 #
-# Copyright (C) 2012-2017 Internet Archive
+# Copyright (C) 2012-2019 Internet Archive
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -21,7 +21,7 @@
 internetarchive.item
 ~~~~~~~~~~~~~~~~~~~~
 
-:copyright: (C) 2012-2017 by Internet Archive.
+:copyright: (C) 2012-2019 by Internet Archive.
 :license: AGPL 3, see LICENSE for more details.
 """
 from __future__ import absolute_import, unicode_literals, print_function
@@ -283,7 +283,8 @@ class Item(BaseItem):
                  ignore_errors=None,
                  on_the_fly=None,
                  return_responses=None,
-                 no_change_timestamp=None):
+                 no_change_timestamp=None,
+                 params=None):
         """Download files from an item.
 
         :param files: (optional) Only download files matching given file names.
@@ -345,6 +346,10 @@ class Item(BaseItem):
                                     current time instead of changing it to that given in
                                     the original archive.
 
+        :type params: dict
+        :param params: (optional) URL parameters to send with
+                       download request (e.g. `cnt=0`).
+
         :rtype: bool
         :returns: True if if all files have been downloaded successfully.
         """
@@ -357,6 +362,7 @@ class Item(BaseItem):
         no_directory = False if no_directory is None else no_directory
         return_responses = False if not return_responses else True
         no_change_timestamp = False if not no_change_timestamp else no_change_timestamp
+        params = None if not params else params
 
         if not dry_run:
             if item_index and verbose is True:
@@ -415,7 +421,7 @@ class Item(BaseItem):
                 continue
             r = f.download(path, verbose, silent, ignore_existing, checksum, destdir,
                            retries, ignore_errors, None, return_responses,
-                           no_change_timestamp)
+                           no_change_timestamp, params)
             if return_responses:
                 responses.append(r)
             if r is False:
@@ -483,12 +489,16 @@ class Item(BaseItem):
         url = '{protocol}//archive.org/metadata/{identifier}'.format(
             protocol=self.session.protocol,
             identifier=self.identifier)
+        if any('/' in k for k in metadata):
+            source_metadata = self.item_metadata
+        else:
+            source_metadata = self.item_metadata.get(target.split('/')[0], {})
         request = MetadataRequest(
             method='POST',
             url=url,
             metadata=metadata,
             headers=self.session.headers,
-            source_metadata=self.item_metadata.get(target.split('/')[0], {}),
+            source_metadata=source_metadata,
             target=target,
             priority=priority,
             access_key=access_key,
@@ -758,12 +768,10 @@ class Item(BaseItem):
         """Upload files to an item. The item will be created if it
         does not exist.
 
-        :type files: list
+        :type files: str, file, list, tuple, dict
         :param files: The filepaths or file-like objects to upload.
 
-        :type kwargs: dict
-        :param kwargs: The keyword arguments from the call to
-                       upload_file().
+        :param \*\*kwargs: Optional arguments that :func:`Item.upload_file()` takes.
 
         Usage::
 
@@ -771,10 +779,32 @@ class Item(BaseItem):
             >>> item = internetarchive.Item('identifier')
             >>> md = dict(mediatype='image', creator='Jake Johnson')
             >>> item.upload('/path/to/image.jpg', metadata=md, queue_derive=False)
-            True
+            [<Response [200]>]
+
+        Uploading multiple files::
+
+            >>> r = item.upload(['file1.txt', 'file2.txt'])
+            >>> r = item.upload([fileobj, fileobj2])
+            >>> r = item.upload(('file1.txt', 'file2.txt'))
+
+        Uploading file objects:
+
+            >>> import io
+            >>> f = io.BytesIO(b"some initial binary data: \\x00\\x01")
+            >>> r = item.upload({'remote-name.txt': f})
+            >>> f = io.BytesIO(b"some more binary data: \\x00\\x01")
+            >>> f.name = 'remote-name.txt'
+            >>> r = item.upload(f)
+
+            *Note: file objects must either have a name attribute, or be uploaded in a
+            dict where the key is the remote-name*
+
+        Setting the remote filename with a dict::
+
+            >>> r = item.upload({'remote-name.txt': '/path/to/local/file.txt'})
 
         :rtype: list
-        :returns: A list of requests.Response objects.
+        :returns: A list of :class:`requests.Response` objects.
         """
         queue_derive = True if queue_derive is None else queue_derive
         remote_dir_name = None
